import logging

logger = logging
logger.basicConfig(
    level=logging.INFO,
    format="%(asctime)s [%(levelname)s] %(message)s",
    handlers=[
        logging.FileHandler("pyland.log"),
        logging.StreamHandler()
    ]
)
    



#!/usr/bin/env python3
import argparse
import importlib
import importlib.util
import importlib.machinery
import sys
import os
from easyland.daemon import Daemon

version = '0.7.0'

def import_from_path(path):
    module_name = os.path.basename(path).replace('-', '_').replace('.py', '')
    spec = importlib.util.spec_from_loader(module_name, importlib.machinery.SourceFileLoader(module_name, path))
    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)
    sys.modules[module_name] = module
    return module

def main():

    parser = argparse.ArgumentParser(description="Easyland - A python swiss-knife to manage Wayland compositors like Hyprland")
# parser.add_argument("-l", "--listen", action="store_true", help="Listen only")
    parser.add_argument("-c", "--config", help="Path to your config file")
    parser.add_argument("-v", "--version", action="store_true", help="Show the version")
    args = parser.parse_args()

    if args.version:
        print('Pyland version: ' + version)
        sys.exit()

    if not args.config:
        print('Please provide a config file with the option -c')
        sys.exit(1)

    config = import_from_path(args.config)
    daemon = Daemon(config)

if __name__ == '__main__':
    main()



from libs.Log import logger
from libs.Config import Config

class Main(Config):
    def on_hyprland_event(self, event, argument):
        logger.info("Hyprland: Receveived '"+event +"' with argument "+argument.strip())        
    
    
    def on_idle(self, time_elapsed):
        logger.info('Current idle time (sec): '+ str(time_elapsed))


    def on_systemd_event(self, sender, signal, payload):
        logger.info("Systemd: Receveived from '"+sender+"': "+ signal +' with payload: '+payload)





from libs.Log import logger
# from libs.Config import Config

# class Main(Config):


listeners = ['hyprland', 'systemd_logind', 'idle']
logger.info('test')
def idle_config():
    return [
        # [2, ['ls'], ['ls /tmp']],
        # [10, ['ls'], ['ls /tmp']],
        [150, ['brightnessctl -s set 0'], ['brightnessctl -r']],
        [600, ['pidof hyprlock || hyprlock']],
        [720, ['hyprctl dispatch dpms off'], ['hyprctl dispatch dpms on']]
    ]

#
# This callback is called when an hyprland IPC event is received
#
# def on_hyprland_event(event, argument):
#     if event in [ "monitoradded", "monitorremoved" ]:
#         logger.info('Handling hyprland event: ' + event)
#         self.set_monitors()

#
# This callback is called when the Systemd event PrepareForSleep is received 
##
#def on_PrepareForSleep(payload):
#    if 'true' in payload:
#        logger.info("Locking the screen before suspend")
        # self.command.shell_command("hyprlock")

#
# Method to set monitors
#
# def set_monitors():
#     logger.info('Setting monitors')
#     if self.command.get_monitor(description="HP 22es") is not None:
#         self.command.hyprctl_command('keyword monitor "eDP-1,disable"')
#     else:
#         self.command.hyprctl_command('keyword monitor "eDP-1,preferred,auto,2"')
#         self.command.shell_command("brightnessctl -s set 0")



import easyland.command as Command 
import subprocess
import threading
import os
from pywayland.client import Display
from pywayland.protocol.wayland.wl_seat import WlSeat
from pywayland.protocol.ext_idle_notify_v1 import (
    ExtIdleNotificationV1,
    ExtIdleNotifierV1
)

class Idle():
    def __init__(self, config):
        self.command = Command.Command()
        self._config = config
        self._display =  Display()
        self._display.connect()
        self._idle_notifier = None
        self._seat = None
        self._notifications = []

    def _global_handler(self, reg, id_num, iface_name, version):
        if iface_name == 'wl_seat':
            self._seat = reg.bind(id_num, WlSeat, version)

        elif iface_name == "ext_idle_notifier_v1":
            self._idle_notifier = reg.bind(id_num, ExtIdleNotifierV1, version)
    
            for idx, c in enumerate(self._config):
                self._notifications.append(None)
                self._notifications[idx] = self._idle_notifier.get_idle_notification(c[0] * 1000, self._seat)
                self._notifications[idx]._index = idx
                self._notifications[idx].dispatcher['idled'] = self._idle_notifier_handler
                self._notifications[idx].dispatcher['resumed'] = self._idle_notifier_resume_handler

    def _idle_notifier_handler(self, notification): 
        for command in self._config[notification._index][1]:
            with open(os.devnull, 'w') as fp:
                subprocess.Popen(command, shell=True, stdout=fp)

    def _idle_notifier_resume_handler(self, notification):
        if len(self._config[notification._index]) > 2:
            for command in self._config[notification._index][2]:
                with open(os.devnull, 'w') as fp:
                    subprocess.Popen(command, shell=True, stdout=fp)

    def setup(self):
        reg = self._display.get_registry()
        reg.dispatcher['global'] = self._global_handler
        while True:
            self._display.dispatch(block=True)





import easyland.command as Command 
import easyland.daemon as Daemon
import easyland.idle as Idle
from easyland.log import logger

class Config:
    def __init__(self):
        self.timeouts = [] 
        # if callable(getattr(self, 'set_idle_config', None)):
        #     self.set_idle_config()

        # self.current_timeout_commands = []
        # self.current_resume_commands = []

        self.command = Command.Command()
        self.daemon = Daemon.Daemon(self, ['hyprland', 'idle', 'systemd'])
        self.setup_idle()
    
    def setup_idle(self):
        if callable(getattr(self, 'idle_config', None)):
           config = self.idle_config()
           idle = Idle.Idle(config)
           idle.setup()




import subprocess
import json
import os
import time
import threading
from easyland.log import logger

class Command():

    def __init__(self):
        pass

    def get_all_monitors(self):
        monitors = self.hyprctl_command("monitors")
        return monitors
    
    def get_monitor(self, name = None, description = None, make = None, model = None):
        monitors = self.get_all_monitors()

        for monitor in monitors:
            if name is not None:
                if name in monitor['name']:
                    return monitor

            if description is not None:
                if description in monitor['description']:
                    return monitor

            if make is not None:
                if make in monitor['make']:
                    return monitor

            if model is not None:
                if model in monitor['model']:
                    return monitor
        return None

    def hyprctl_command(self, command):
        cmd = "hyprctl -j " + command
        logger.info("Executing command: "+cmd) 
        output = subprocess.check_output(cmd, shell=True)
        decoded_output = output.decode("utf-8")
        
        try:
            json_output = json.loads(decoded_output)
        except json.decoder.JSONDecodeError as e:
            return decoded_output
        return json_output
    
    def shell_command(self, command):
        command = command.strip()
        if command.endswith('&'):
            command = command[:-1]
            logger.info("Executing background command: "+command)
            with open(os.devnull, 'w') as fp:
                subprocess.Popen(command, shell=True, stdout=fp)
        else:
            logger.info("Executing command: "+command) 
            output = subprocess.check_output(command, shell=True)
            decoded_output = output.decode("utf-8")
            return decoded_output





import subprocess
import threading
import re
import time
import sys
from easyland.log import logger

class Daemon():

    def __init__(self, config):
        self.config = config 
        listeners = self.get_listeners()
        logger.info('Starting easyland daemon')
        # self.last_event_time = time.time()
        # if 'hyprland' in watchers:
        #     hyprland_thread = threading.Thread(target=self.launch_hyprland_daemon)
        #     hyprland_thread.start()
        # if 'systemd' in watchers:
        #     systemd_thread = threading.Thread(target=self.launch_systemd_login_daemon)
        #     systemd_thread.start()
    
    def get_listeners(self):
        if hasattr(self.config, 'listeners'):
            return self.config.listeners
        else:
            logger.error('No listeners defined in the config file')
            sys.exit(1)


    def call_handler(self, handler, *argv): 
        func = getattr(self.config, handler, None)
        if callable(func):
            func(*argv)
    
    def launch_hyprland_daemon(self):
        logger.info('Launching hyprland daemon')
        cmd = "socat -U - UNIX-CONNECT:/tmp/hypr/$HYPRLAND_INSTANCE_SIGNATURE/.socket2.sock"
        ps = subprocess.Popen(cmd,shell=True,stdout=subprocess.PIPE,stderr=subprocess.STDOUT)
        while True:
            for line in iter(ps.stdout.readline, ""):
                self.last_event_time = time.time()
                decoded_line = line.decode("utf-8")
                if '>>' in decoded_line:
                    data = decoded_line.split('>>')
                    self.call_handler('on_hyprland_event', data[0], data[1])

    def launch_systemd_login_daemon(self):
        logger.info('Launching systemd daemon')
        cmd = "gdbus monitor --system --dest org.freedesktop.login1"
        ps = subprocess.Popen(cmd,shell=True,stdout=subprocess.PIPE,stderr=subprocess.STDOUT)
        while True:
            for line in iter(ps.stdout.readline, ""):
                decoded_line = line.decode("utf-8").strip()
                res = re.match(r'(.+?): ([^\s]+?) \((.*?)\)$', decoded_line)
                if res:
                    sender = res.group(1)
                    name = res.group(2)
                    payload = res.group(3)

                    if 'Properties' not in name:
                        signal_name = name.split('.')[-1]
                        f = 'on_' + signal_name 
                        self.call_handler(f, payload)
                        self.call_handler('on_systemd_event', sender, signal_name, payload)
   

